import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import "core-js/modules/es6.string.anchor";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React, { useState, useContext } from 'react';
import ActionButton from './ActionButton';
import Drawer from './Drawer';
import AmpDrawer from './amp/AmpDrawer';
import AppContext from './AppContext';
import PropTypes from 'prop-types';
/**
 *
 * Example use:
 *
 * ```js
 * <DrawerButton ampStateId="sizeChart" label="Open Size Chart">
 *   <CmsSlot>{product.sizeChartHtml}</CmsSlot>
 * </DrawerButton>
 * ```
 *
 * The children components are rendered within the drawer content.
 *
 * The `anchor` prop controls which side from which the drawer will appear.
 *
 * You can pass a function as the children of the DrawerButton in order to render a custom
 * close button. You just need to spread the passed argument onto the button's props to
 * handle the toggling of the drawer.
 *
 * And the rest of the `DrawerButton` props are spread to the `ActionButton` which triggers the drawer to open.
 *
 */

export default function DrawerButton(_ref) {
  var ampStateId = _ref.ampStateId,
      anchor = _ref.anchor,
      showCloseButton = _ref.showCloseButton,
      closeButtonProps = _ref.closeButtonProps,
      children = _ref.children,
      props = _objectWithoutProperties(_ref, ["ampStateId", "anchor", "showCloseButton", "closeButtonProps", "children"]);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      open = _useState2[0],
      setOpen = _useState2[1];

  var toggle = function toggle() {
    return setOpen(!open);
  };

  var _useContext = useContext(AppContext),
      amp = _useContext.app.amp;

  var drawerProps = {
    showCloseButton: showCloseButton,
    closeButtonProps: closeButtonProps,
    anchor: anchor // This should be spread over any custom close buttons used within the content

  };
  var customCloseButtonProps = {
    onClick: toggle,
    role: 'button',
    tabIndex: '0',
    on: "tap:AMP.setState({ ".concat(ampStateId, ": { open: false } })")
  };
  children = typeof children === 'function' ? children(customCloseButtonProps) : children;

  if (amp) {
    var triggerOpen = anchor === 'left' || anchor === 'right' ? // Triggering sidebar to open and a separate state to control the backdrop
    "tap:".concat(ampStateId, ".open,AMP.setState({ ").concat(ampStateId, ": { open: true } })") : "tap:AMP.setState({ ".concat(ampStateId, ": { open: true } })");
    return React.createElement(React.Fragment, null, React.createElement(ActionButton, _extends({}, props, {
      on: triggerOpen
    })), React.createElement(AmpDrawer, _extends({
      ampStateId: ampStateId
    }, drawerProps), children));
  }

  return React.createElement(React.Fragment, null, React.createElement(ActionButton, _extends({}, props, {
    onClick: toggle
  })), React.createElement(Drawer, _extends({
    open: open,
    onRequestClose: toggle
  }, drawerProps), children));
}
DrawerButton.propTypes = {
  /**
   * The side from which the drawer will slide in.
   */
  anchor: PropTypes.oneOf(['left', 'top', 'right', 'bottom']),

  /**
   * Set to `false` to hide the close button.
   */
  showCloseButton: PropTypes.bool,

  /**
   * Props to pass to the close button.
   */
  closeButtonProps: PropTypes.object
};
DrawerButton.defaultProps = {
  anchor: 'bottom',
  showCloseButton: true,
  ampStateId: 'drawerButton'
};
//# sourceMappingURL=DrawerButton.js.map