import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import "core-js/modules/es6.regexp.search";

var _dec, _dec2, _class, _class2, _temp;

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React, { Component, Fragment } from 'react';
import withStyles from '@material-ui/core/styles/withStyles';
import Button from '@material-ui/core/Button';
import Fab from '@material-ui/core/Fab';
import Input from '@material-ui/core/Input';
import IconButton from '@material-ui/core/IconButton';
import SearchIcon from '@material-ui/icons/Search';
import ClearIcon from '@material-ui/icons/Clear';
import Typography from '@material-ui/core/Typography';
import PropTypes from 'prop-types';
import Highlight from 'react-highlighter';
import CircularProgress from '@material-ui/core/CircularProgress';
import Link from './Link';
import { inject, observer } from 'mobx-react';
import { onPatch } from 'mobx-state-tree';
import Image from './Image';
import Container from './Container';
import classnames from 'classnames';
import Drawer from '@material-ui/core/Drawer';
import { Hbox } from './Box';
import Track from './Track';
import AmpSearchDrawer from './amp/AmpSearchDrawer';
import AmpSearchResults from './amp/AmpSearchResults';
import AmpForm from './amp/AmpForm';
import AmpState from './amp/AmpState';
/**
 * A modal search UI that displays a single text search field and grouped results.  The
 * data for this component is defined in react-storefront/model/SearchModelBase.  In most cases, you can
 * add this component to your PWA simply by adding it to App.js without any props:
 *
 *  <SearchDrawer/>
 *
 * When the user enters text in the search field, SearchModelBase calls /search/suggest, which by default is mapped to
 * src/search/suggest-handler.js in the starter app.
 *
 * See https://github.com/moovweb/react-storefront-boilerplate/tree/master/src/search/suggest-handler.js for
 * the placeholder implementation of the suggestion API.
 *
 * When the user taps the search icon or types the enter key in the search field, the drawer closes and the url
 * is set to /search?q=(the user's search text).
 *
 * See src/routes.js to edit the mappings for /search and /search/suggest.
 */

export var styles = function styles(theme) {
  return {
    root: {
      zIndex: 9999
    },
    paper: {
      backgroundColor: 'rgba(255, 255, 255, .7)'
    },
    paperAnchorBottom: {
      top: '0'
    },
    wrap: {
      height: '100vh',
      display: 'flex',
      flexDirection: 'column',
      alignItems: 'stretch'
    },
    header: {
      backgroundColor: theme.palette.primary.main,
      padding: theme.margins.container,
      display: 'flex',
      flexDirection: 'column',
      alignItems: 'stretch'
    },
    closeButton: {
      color: theme.palette.primary.contrastText,
      margin: '-10px -10px 10px 0',
      alignSelf: 'flex-end',
      '& span': {
        textTransform: 'uppercase',
        fontWeight: 'bold'
      }
    },
    closeButtonText: {
      border: "1px solid ".concat(theme.palette.primary.contrastText),
      margin: '0 0 10px 0'
    },
    searchField: {
      flexGrow: 1,
      border: 0,
      borderRadius: '35px',
      backgroundColor: theme.palette.background.paper,
      margin: 0,
      height: '48px'
    },
    searchInput: {
      padding: '0 0 0 20px'
    },
    groupCaption: {
      textTransform: 'uppercase',
      margin: '30px 0 10px 0'
    },
    group: {
      listStyle: 'none',
      margin: 0,
      padding: 0,
      '& a strong': {
        fontWeight: 'bold',
        color: theme.palette.text.primary
      }
    },
    thumbnailGroup: {
      display: 'flex',
      listStyle: 'none',
      margin: '0 -15px',
      padding: '0 10px',
      overflowX: 'auto',
      '& > li': {
        margin: '5px'
      },
      '& img': {
        height: '120px'
      }
    },
    thumbnail: {
      marginBottom: '10px'
    },
    results: {
      position: 'relative',
      // TODO: Only set for AMP if this affects PWA
      flex: 1,
      overflowY: 'auto'
    },
    loading: {
      display: 'flex',
      flex: 1,
      alignItems: 'center',
      justifyContent: 'center',
      height: '100%'
    },
    searchFab: {
      height: '48px',
      width: '48px',
      marginLeft: '10px',
      backgroundColor: theme.palette.background.paper,
      color: theme.palette.text.secondary
    },
    hidden: {
      display: 'none'
    }
  };
};
var SearchDrawer = (_dec = withStyles(styles, {
  name: 'RSFSearchDrawer'
}), _dec2 = inject(function (_ref) {
  var _ref$app = _ref.app,
      search = _ref$app.search,
      amp = _ref$app.amp,
      history = _ref.history,
      theme = _ref.theme;
  return {
    search: search,
    history: history,
    theme: theme,
    amp: amp
  };
}), _dec(_class = _dec2(_class = observer(_class = (_temp = _class2 =
/*#__PURE__*/
function (_Component) {
  _inherits(SearchDrawer, _Component);

  function SearchDrawer(_ref2) {
    var _this;

    var _search = _ref2.search;

    _classCallCheck(this, SearchDrawer);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(SearchDrawer).call(this));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onSearchPatch", function (_ref3) {
      var path = _ref3.path,
          value = _ref3.value;
      var blurBackground = _this.props.blurBackground;

      if (path === '/show') {
        if (value) {
          setTimeout(function () {
            return _this.input.focus();
          }, 250);
        } else {
          _this.input.blur();
        }

        if (blurBackground) {
          document.body.classList.toggle('moov-blur', value);
        }
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onChangeSearchText", function (text) {
      _this.props.search.setText(text);
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "clearSearch", function () {
      _this.onChangeSearchText('');

      _this.input.focus();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "hide", function () {
      _this.props.search.toggle(false);
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onSearchSubmit", function () {
      var _this$props = _this.props,
          searchURL = _this$props.searchURL,
          searchFieldName = _this$props.searchFieldName,
          search = _this$props.search;
      var separator = searchURL.indexOf('?') === -1 ? '?' : '&';
      var url = "".concat(searchURL).concat(separator).concat(searchFieldName, "=").concat(encodeURIComponent(search.text));

      _this.props.history.push(url);

      _this.hide();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onSearchFormSubmit", function (e) {
      e.preventDefault();

      _this.onSearchSubmit();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "onInputFocus", function () {
      var _assertThisInitialize = _assertThisInitialized(_assertThisInitialized(_this)),
          input = _assertThisInitialize.input;

      input.setSelectionRange(0, input.value.length);
    });

    _this.inputRef = function (input) {
      return _this.input = input;
    };

    onPatch(_search, _this.onSearchPatch);
    return _this;
  }

  _createClass(SearchDrawer, [{
    key: "renderContent",
    value: function renderContent() {
      var _this2 = this;

      var _this$props2 = this.props,
          classes = _this$props2.classes,
          search = _this$props2.search,
          placeholder = _this$props2.placeholder,
          searchButtonVariant = _this$props2.searchButtonVariant,
          showClearButton = _this$props2.showClearButton,
          searchURL = _this$props2.searchURL,
          searchFieldName = _this$props2.searchFieldName,
          amp = _this$props2.amp,
          inputProps = _this$props2.inputProps;

      var HideWhenEmpty = function HideWhenEmpty(_ref4) {
        var children = _ref4.children;
        return React.createElement("div", {
          className: search.text.trim().length ? null : classes.hidden,
          "amp-bind": "class=>rsfSearchDrawer.searchText.length > 0 ? \"\" : \"".concat(classes.hidden, "\"")
        }, children);
      };

      var SearchButton = function SearchButton(_ref5) {
        var Component = _ref5.Component,
            others = _objectWithoutProperties(_ref5, ["Component"]);

        return React.createElement(Component, _extends({
          rel: "search",
          type: "submit",
          disabled: !amp && search.text.trim().length === 0
        }, others), React.createElement(SearchIcon, null));
      };

      return React.createElement("div", {
        className: classes.wrap
      }, React.createElement(AmpState, {
        id: "rsfSearchDrawer",
        initialState: {
          open: false,
          searchText: ''
        }
      }, React.createElement(Track, {
        event: "searchSubmitted",
        trigger: "onSubmit",
        term: search.text
      }, React.createElement(AmpForm, {
        onSubmit: this.onSearchFormSubmit,
        action: searchURL,
        mask: false
      }, React.createElement("div", {
        className: classes.header
      }, this.renderCloseButton(), React.createElement(Hbox, null, React.createElement(Input, {
        type: "text",
        name: searchFieldName,
        value: search.text,
        placeholder: placeholder,
        onFocus: this.onInputFocus,
        onChange: function onChange(e) {
          return _this2.onChangeSearchText(e.target.value);
        },
        inputProps: _objectSpread({
          'amp-bind': 'value=>rsfSearchDrawer.searchText'
        }, inputProps),
        on: "input-debounced:AMP.setState({ rsfSearchDrawer: { searchText: rsfSearchDrawer.___moov_submitting ? rsfSearchDrawer.searchText : event.value } })",
        disableUnderline: true,
        inputRef: this.inputRef,
        classes: {
          root: classes.searchField,
          input: classes.searchInput
        },
        endAdornment: showClearButton ? React.createElement(HideWhenEmpty, null, React.createElement(IconButton, {
          onClick: this.clearSearch,
          className: classes.searchReset,
          rel: "clear",
          on: "tap:AMP.setState({ rsfSearchDrawer: { searchText: '' }})"
        }, React.createElement(ClearIcon, {
          rel: "clear"
        }))) : searchButtonVariant === 'icon' && React.createElement(SearchButton, {
          Component: Button,
          className: classes.searchButton
        })
      }), searchButtonVariant === 'fab' && React.createElement(HideWhenEmpty, null, React.createElement(SearchButton, {
        Component: Fab,
        className: classes.searchFab
      }))))))), search.loading && React.createElement("div", {
        className: classes.loading
      }, React.createElement(CircularProgress, null)), this.renderResults());
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props3 = this.props,
          classes = _this$props3.classes,
          search = _this$props3.search,
          blurBackground = _this$props3.blurBackground,
          amp = _this$props3.amp,
          drawerProps = _this$props3.drawerProps;

      if (amp) {
        return React.createElement(AmpSearchDrawer, null, this.renderContent());
      } else {
        return React.createElement(Drawer, _extends({
          open: search.show,
          anchor: "bottom",
          className: classes.root,
          classes: {
            paper: blurBackground ? classes.paper : '',
            paperAnchorBottom: classes.paperAnchorBottom
          }
        }, drawerProps), this.renderContent());
      }
    }
  }, {
    key: "renderResults",
    value: function renderResults() {
      var _this3 = this;

      var _this$props4 = this.props,
          _this$props4$search = _this$props4.search,
          loading = _this$props4$search.loading,
          results = _this$props4$search.results,
          classes = _this$props4.classes,
          initialContent = _this$props4.initialContent,
          amp = _this$props4.amp,
          ampThumbnailHeight = _this$props4.ampThumbnailHeight,
          ampThumbnailWidth = _this$props4.ampThumbnailWidth;

      if (amp) {
        return React.createElement(AmpSearchResults, {
          classes: classes,
          ampThumbnailHeight: ampThumbnailHeight,
          ampThumbnailWidth: ampThumbnailWidth
        });
      }

      if (loading) {
        return null;
      } else if (results && results.length) {
        return React.createElement("div", {
          className: classes.results
        }, results.map(function (group) {
          return React.createElement(Container, {
            key: group.caption
          }, React.createElement(Typography, {
            className: classes.groupCaption
          }, group.caption), _this3.renderGroup(group));
        }));
      } else {
        return initialContent;
      }
    }
  }, {
    key: "renderCloseButton",
    value: function renderCloseButton() {
      var _classnames;

      var _this$props5 = this.props,
          classes = _this$props5.classes,
          closeButtonText = _this$props5.closeButtonText,
          CloseButtonIcon = _this$props5.CloseButtonIcon;
      var ButtonElement = closeButtonText ? Button : IconButton;
      return React.createElement(ButtonElement, {
        className: classnames((_classnames = {}, _defineProperty(_classnames, classes.closeButton, true), _defineProperty(_classnames, classes.closeButtonText, closeButtonText != null), _classnames)),
        color: "primary",
        onClick: this.hide,
        on: "tap:AMP.setState({ rsfSearchDrawer: { open: false } })"
      }, closeButtonText || React.createElement(CloseButtonIcon, null));
    }
  }, {
    key: "renderGroup",
    value: function renderGroup(group) {
      var _classnames2,
          _this4 = this;

      var classes = this.props.classes;
      return React.createElement("ul", {
        className: classnames((_classnames2 = {}, _defineProperty(_classnames2, classes.group, !group.thumbnails), _defineProperty(_classnames2, classes.thumbnailGroup, group.thumbnails), _classnames2))
      }, group.results.map(function (result, i) {
        return React.createElement("li", {
          key: i
        }, React.createElement(Track, {
          event: "searchSubmitted",
          term: _this4.props.search.text
        }, React.createElement(Link, {
          to: result.url,
          onClick: _this4.hide
        }, group.thumbnails ? _this4.renderThumbnail(result) : _this4.renderLinkText(result))));
      }));
    }
  }, {
    key: "renderLinkText",
    value: function renderLinkText(result) {
      return React.createElement(Typography, null, React.createElement(Highlight, {
        search: this.props.search.text
      }, result.text));
    }
  }, {
    key: "renderThumbnail",
    value: function renderThumbnail(result) {
      var classes = this.props.classes;
      return React.createElement(Fragment, null, React.createElement("div", null, React.createElement(Image, {
        className: classes.thumbnail,
        src: result.thumbnail,
        height: result.thumbnailHeight,
        width: result.thumbnailWidth
      })), React.createElement("div", null, React.createElement(Typography, null, result.text)));
    }
    /**
     * Updates the model when the user enteres search text
     */

  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      if (this.props.blurBackground && this.props.search.show) {
        document.body.classList.remove('moov-blur');
      }
    }
  }]);

  return SearchDrawer;
}(Component), _defineProperty(_class2, "propTypes", {
  /**
   * The placeholder text for the input.  Defaults to "Search..."
   */
  placeholder: PropTypes.string,

  /**
   * Set this prop to use a text button instead of an icon for the close button.  If set, CloseButtonIcon
   * will be ignored
   */
  closeButtonText: PropTypes.string,

  /**
   * Overrides the default close icon.  Takes a React component.
   */
  CloseButtonIcon: PropTypes.func,

  /**
   * Set to false to disable background blurring.  Defaults to true.
   */
  blurBackground: PropTypes.bool,

  /**
   * Set to "icon" to display the search button as an icon in the search input.
   * Set to "fab" to display the search button as a separate floating action button to the right of the search input when
   * the user enters text.
   */
  searchButtonVariant: PropTypes.oneOf(['icon', 'fab']),

  /**
   * Set to false to never show the clear button.  The search icon will be shown inside the input even when the user has entered text.
   */
  showClearButton: PropTypes.bool,

  /**
   * Content to display in place of the results when the search is blank.
   */
  initialContent: PropTypes.element,

  /**
   * The URL to which the search is submitted.
   */
  searchURL: PropTypes.string,

  /**
   * A name for the search field, which will be the name of the query string parameter
   * through which the search string is submitted.
   */
  searchFieldName: PropTypes.string,

  /**
   * AMP Thumbnail Image Width. Defaults to 120
   */
  ampThumbnailWidth: PropTypes.number,

  /**
   * AMP Thumbnail Image Height. Defaults to 120
   */
  ampThumbnailHeight: PropTypes.number,

  /**
   * Props to be applied to the underlying Drawer component
   */
  drawerProps: PropTypes.object,

  /**
   * Props to be applied to the underlying Input component
   */
  inputProps: PropTypes.object
}), _defineProperty(_class2, "defaultProps", {
  placeholder: 'Search...',
  CloseButtonIcon: function CloseButtonIcon() {
    return React.createElement(ClearIcon, null);
  },
  blurBackground: true,
  searchButtonVariant: 'fab',
  searchURL: '/search',
  searchFieldName: 'q',
  showClearButton: true,
  ampThumbnailWidth: 120,
  ampThumbnailHeight: 120,
  drawerProps: {
    ModalProps: {
      hideBackdrop: true
    }
  }
}), _temp)) || _class) || _class) || _class);
export { SearchDrawer as default };
//# sourceMappingURL=SearchDrawer.js.map