import _objectSpread from "@babel/runtime/helpers/objectSpread";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.object.keys";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
import qs from 'qs';
/**
 * The old image optimizer intented for mobile phones only
 */

export var LEGACY_IMAGE_SERVICE = 'https://opt.moovweb.net';
/**
 * The new image optimizer which supports all device types.
 */

export var XDN_IMAGE_SERVICE = 'https://optimize.moovweb.net';
/**
 * The new image optimizer running on localhost:8090
 */

export var LOCAL_IMAGE_SERVICE = 'http://localhost:8090';
var optimizerUrlBase = LEGACY_IMAGE_SERVICE;
/**
 * Changes the base URL for the Moovweb image service.
 * @param {String} url Use `LEGACY_IMAGE_OPTIMIZER` or `RESPONSIVE_IMAGE_OPTIMIZER`.
 * @param {Object} options
 * @param {Object} options.version The version of the image optimizer API to use. Defaults to '1'
 */

export function configureImageService(url) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$version = _ref.version,
      version = _ref$version === void 0 ? '1' : _ref$version;

  optimizerUrlBase = url;

  if (url === XDN_IMAGE_SERVICE) {
    optimizerUrlBase += "/v".concat(version);
  }
}
/**
 * Resets the image service to the default implementation
 */

export function resetImageService() {
  optimizerUrlBase = LEGACY_IMAGE_SERVICE;
}
/**
 * Creates an optimized image URL
 * @param {String} src
 * @param {Object} options
 * @param {Number} options.height The max height of the image when served to phones
 * @param {Number} options.width The max width of the image when served to phones
 * @param {Number} options.quality A number from 1-100 representing the amount to downscale the source image when served to phones
 * @param {Object} options.format "jpeg" or "webp" If webp is specified, webp will only be served to browsers that support it.
 * @return {String}
 */

export function createOptimizedSrc(src) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  // The legacy service used fmt, so we kept it for the new responsive service,
  // but we use "format" here because non of the other options are abbreviated
  if (options.format) options.fmt = options.format;

  if (Object.keys(options).length > 0) {
    return "".concat(optimizerUrlBase, "/img?").concat(qs.stringify(_objectSpread({}, options, {
      img: src
    })));
  } else {
    return src;
  }
}
//# sourceMappingURL=imageService.js.map