import _regeneratorRuntime from "@babel/runtime/regenerator";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
import { useEffect, useContext, useRef } from 'react';
import AppContext from '../AppContext';
import { reaction } from 'mobx';
import { PageContext } from '../Pages';
/**
 * A hook for fetching late-loaded personalized data.  This hook automatically
 * calls the `loadPersonalization()` method on the model in the specified `branch` in the app
 * state tree whenever the model's id field changes.
 *
 * Example
 *
 * ```js
 * function Product() {
 *   usePersonalization(app => app.product)
 *   return //...
 * }
 * ```
 *
 * @param {Function} branch A function that returns the model that should fetch personalized data.
 * @returns {Function}
 */

export default function usePersonalization(branch) {
  var _useContext = useContext(AppContext),
      app = _useContext.app;

  var pageContext = useContext(PageContext);
  var id = useRef(null);

  var loadPersonalization =
  /*#__PURE__*/
  function () {
    var _ref = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee(_ref2) {
      var loading, page, model;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              loading = _ref2.loading, page = _ref2.page, model = _ref2.model;

              if (!(model == null)) {
                _context.next = 5;
                break;
              }

              return _context.abrupt("return");

            case 5:
              if (model.loadPersonalization) {
                if (pageContext === page && !loading) {
                  id.current = model.id;
                  model.loadPersonalization();
                }
              } else {
                console.warn("The model at ".concat(branch, " does not implement loadPersonalization.  You should implement this action so that usePersonalization/withPersonalization can load personalized data from the server."));
              }

            case 6:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function loadPersonalization(_x) {
      return _ref.apply(this, arguments);
    };
  }();

  var shouldFetchPersonalization = function shouldFetchPersonalization() {
    return {
      loading: app.loading,
      page: app.page,
      model: branch(app)
    };
  };

  useEffect(function () {
    // check if we should load personalization data immediately as is the case if
    // the user initially lands on a page with usePersonalization
    loadPersonalization(shouldFetchPersonalization()); // it is critical that we return the reaction disposer here (returned by reaction)
    // so that the reaction is disposed when the component unmounts

    return reaction(shouldFetchPersonalization, loadPersonalization);
  }, []);
}
//# sourceMappingURL=usePersonalization.js.map