import _regeneratorRuntime from "@babel/runtime/regenerator";
import "core-js/modules/es6.regexp.to-string";
import "regenerator-runtime/runtime";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import "core-js/modules/es6.string.iterator";
import "core-js/modules/es6.map";
import "core-js/modules/es6.regexp.match";
import "core-js/modules/es6.regexp.split";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.object.keys";
import "core-js/modules/es6.string.ends-with";
import "core-js/modules/es6.array.find";
import "core-js/modules/es6.regexp.replace";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
import React from 'react';
import { renderToString } from 'react-dom/server';
import { Provider } from 'mobx-react';
import { SheetsRegistry } from 'react-jss/lib/jss';
import JssProvider from 'react-jss/lib/JssProvider';
import { MuiThemeProvider } from '@material-ui/core/styles';
import createGenerateClassName from './utils/createGenerateClassName';
import minifyStyles from './utils/minifyStyles'; // Convert mobx-state-tree model to json and escape <\ in a closing script tag to avoid untimely script closing.

var getSanitizedModelJson = function getSanitizedModelJson(state) {
  return sanitizeJsonForScript(state.toJSON());
}; // Escape <\ in a closing script tag to avoid untimely script closing


var sanitizeJsonForScript = function sanitizeJsonForScript(state) {
  return JSON.stringify(state || {}).replace(/<\/script/gi, '<\\/script');
};
/**
 * Get javascript asset filename by chunk name
 * @private
 * @param  {Object} stats Webpack generated stats
 * @param  {String} chunk Chunk name
 * @return {String}       Asset filename
 */


export function getJS(stats, chunk) {
  var assets = stats.assetsByChunkName[chunk];

  if (!assets) {
    return null;
  } else if (Array.isArray(assets)) {
    return assets.find(function (file) {
      return file.endsWith('.js');
    });
  } else {
    return assets;
  }
}
/**
 * Gets all source paths for the given chunk
 * @private
 * @param  {Object} stats Webpack generated stats
 * @param  {String} chunk Chunk name
 * @return {String[]}
 */

function getSources(stats, chunk) {
  return Object.keys(stats.assetsByChunkName).filter(function (key) {
    // render any that have `chunk` in the name, split by ~
    // when webpack sees two or more chunks that share some vendor libs, it can decide
    // to create a separate chunk just for the libs those two chunks share.
    // for example: vendors~component-Component~guide-Guide
    return key.split('~').some(function (page) {
      return page === chunk;
    });
  }).map(function (key) {
    var assets = stats.assetsByChunkName[key];

    if (Array.isArray(assets)) {
      // we'll get an array when building with source maps, in that case pick the JS file, not the map
      return stats.assetsByChunkName[key].find(function (file) {
        return !file.match(/\.map$/);
      });
    } else {
      // otherwise the value will be the JS file
      return assets;
    }
  });
}
/**
 * Render HTML of given component
 * @private
 * @param  {options} options
 * @param  {React.Component} options.component Component to be rendered
 * @param  {Object} options.providers          Data providers
 * @param  {Object} options.registry           JSS Sheets Registry
 * @param  {Object} options.theme              MUI Theme
 * @param  {Object} options.cssPrefix          A prefix to apply to css classes
 * @return {String}                            HTML
 */


export function renderHtml(_ref) {
  var component = _ref.component,
      providers = _ref.providers,
      registry = _ref.registry,
      theme = _ref.theme,
      _ref$cssPrefix = _ref.cssPrefix,
      cssPrefix = _ref$cssPrefix === void 0 ? 'jss' : _ref$cssPrefix;
  return renderToString(React.createElement(Provider, providers, React.createElement(JssProvider, {
    classNamePrefix: cssPrefix,
    registry: registry,
    generateClassName: createGenerateClassName()
  }, React.createElement(MuiThemeProvider, {
    theme: theme,
    sheetsManager: new Map()
  }, component))));
}
/**
 * Renders initial state for client side hydration
 * @private
 * @param  {options} options                      Model instance
 * @param  {Object} options.state                 The initial app state needed for hydration
 * @param  {Object} options.routeData             The data that resulted from the route being run.  This is needed for creating a cache
 *                                                entry for the equivalend .json request if the user navigates back to this page.
 * @param  {Boolean} options.defer                Should defer execution
 * @param  {String} options.initialStateProperty  Property name on window object
 * @return {String}                               Script HTML
 */

export function renderInitialStateScript(_ref2) {
  var state = _ref2.state,
      routeData = _ref2.routeData,
      defer = _ref2.defer,
      _ref2$initialStatePro = _ref2.initialStateProperty,
      initialStateProperty = _ref2$initialStatePro === void 0 ? 'initialState' : _ref2$initialStatePro,
      _ref2$initialRouteDat = _ref2.initialRouteDataProperty,
      initialRouteDataProperty = _ref2$initialRouteDat === void 0 ? 'initialRouteData' : _ref2$initialRouteDat;
  return "<script type=\"text/javascript\" ".concat(defer ? 'defer' : '', ">\n    window.").concat(initialStateProperty, "=Object.freeze(").concat(getSanitizedModelJson(state), ")\n    window.").concat(initialRouteDataProperty, "=Object.freeze(").concat(sanitizeJsonForScript(routeData), ")\n\t</script>");
}
/**
 * Renders script for a specified chunk
 * @private
 * @param  {String} src             Source of script
 * @param  {Boolean} defer          Should defer execution
 * @return {String}                 Script HTML
 */

export function renderScript(src, defer) {
  return "<script type=\"text/javascript\" ".concat(defer ? 'defer' : '', " src=\"").concat(src, "\"></script>");
}
/**
 * Renders extracted CSS from Sheets Registry
 * @private
 * @param  {options} options
 * @param  {Object} options.registry  JSS Sheets Registry
 * @param  {String} options.id        ID for style tag
 * @return {String}                   Style HTML
 */

export function renderStyle(_x) {
  return _renderStyle.apply(this, arguments);
}
/**
 * Extracts scripts from sources in chunk
 * @private
 * @param {Object} options
 * @param {Object} options.stats    Webpack generated stats
 * @param {String} options.chunk     Chunk name
 */

function _renderStyle() {
  _renderStyle = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(_ref4) {
    var registry, _ref4$id, id, minify, css;

    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            registry = _ref4.registry, _ref4$id = _ref4.id, id = _ref4$id === void 0 ? 'ssr-css-jss' : _ref4$id, minify = _ref4.minify;
            css = registry.toString(); // css might be undefined, e.g. after an error.

            if (!(css && minify)) {
              _context.next = 6;
              break;
            }

            _context.next = 5;
            return minifyStyles(css);

          case 5:
            css = _context.sent;

          case 6:
            return _context.abrupt("return", "<style id=\"".concat(id, "\">").concat(css, "</style>"));

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _renderStyle.apply(this, arguments);
}

export function getScripts(_ref3) {
  var stats = _ref3.stats,
      chunk = _ref3.chunk;
  var assetPathBase = process.env.ASSET_PATH_BASE || '';
  return getSources(stats, chunk).map(function (src) {
    return "".concat(assetPathBase, "/pwa/").concat(src);
  });
}
/**
 * Renders a link prefetch header value
 * @private
 * @param {String} src Source of script
 */

export function renderPreloadHeader(src) {
  return "<".concat(src, ">; rel=preload; as=script");
}
/**
 * Helper for extracting location object from Adapt environment
 * @private
 * @param  {Object} env Adapt Environment
 * @return {Object}     Location Object
 */

function getLocation(env) {
  var _env$path$split = env.path.split('?'),
      _env$path$split2 = _slicedToArray(_env$path$split, 2),
      pathname = _env$path$split2[0],
      search = _env$path$split2[1];

  return {
    protocol: env.secure ? 'https' : 'http',
    hostname: env.host,
    pathname: pathname,
    search: search || ''
  };
}
/**
 * Renders a component on the server.
 * @private
 * @param  {options} options
 * @param  {React.Component} options.component    Component to be rendered
 * @param  {Object} options.state                 Model instance
 * @param  {Object} options.theme                 MUI Theme
 * @param  {Object} options.stats                 Webpack generated stats
 * @param  {String} options.clientChunk           Chunk name for hydration script
 * @param  {String} options.initialStateProperty  Optional window property name for initial state
 * @param  {Boolean} options.injectAssets          Defaults to true.  Set this to false to prevent scripts and css from automatically being injected into the document.
 * @param  {String} options.cssPrefix            A prefix to apply to css class names
 * @param  {Boolean} options.minifyStyles         Whether to apply minification to styles. Defaults to false.
 * @return {Object}                               Components for SSR
 */


export function render(_x2) {
  return _render.apply(this, arguments);
}

function _render() {
  _render = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(_ref5) {
    var component, state, theme, stats, clientChunk, initialStateProperty, _ref5$injectAssets, injectAssets, _ref5$minifyStyles, minifyStyles, _ref5$cssPrefix, cssPrefix, registry, html, scripts, result;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            component = _ref5.component, state = _ref5.state, theme = _ref5.theme, stats = _ref5.stats, clientChunk = _ref5.clientChunk, initialStateProperty = _ref5.initialStateProperty, _ref5$injectAssets = _ref5.injectAssets, injectAssets = _ref5$injectAssets === void 0 ? true : _ref5$injectAssets, _ref5$minifyStyles = _ref5.minifyStyles, minifyStyles = _ref5$minifyStyles === void 0 ? false : _ref5$minifyStyles, _ref5$cssPrefix = _ref5.cssPrefix, cssPrefix = _ref5$cssPrefix === void 0 ? '' : _ref5$cssPrefix;
            registry = new SheetsRegistry();
            state.applyState({
              location: getLocation(env)
            }, 'REPLACE');
            html = renderHtml({
              component: component,
              providers: {
                app: state
              },
              registry: registry,
              theme: theme,
              cssPrefix: cssPrefix
            });
            scripts = getScripts({
              stats: stats,
              chunk: clientChunk
            });
            _context2.t0 = html;
            _context2.next = 8;
            return renderStyle({
              id: "ssr-css-".concat(cssPrefix),
              registry: registry,
              minify: minifyStyles
            });

          case 8:
            _context2.t1 = _context2.sent;
            _context2.t2 = renderInitialStateScript({
              state: state,
              defer: false,
              initialStateProperty: initialStateProperty
            });
            _context2.t3 = scripts.map(renderScript);
            result = {
              html: _context2.t0,
              style: _context2.t1,
              initialStateScript: _context2.t2,
              componentScript: _context2.t3
            };

            if (injectAssets) {
              $head.append(result.style);
              $body.append(result.initialStateScript);
              scripts.forEach(function (src) {
                if (!$body.find("script[src=\"".concat(src, "\"]")).length) {
                  $body.append(renderScript(src));
                }
              });
            }

            return _context2.abrupt("return", result);

          case 14:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));
  return _render.apply(this, arguments);
}
//# sourceMappingURL=renderers.js.map